#!/bin/bash
#

# build_geant4.sh <product_directory> <e9|e7|e9:qt|e7:qt> <debug|opt|prof> [tar]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e9|e7|e10:qt|e9:qt|e7:qt> <debug|opt|prof> [tar]"
   echo "       NOTE: The qt build is only supported for slf6"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	build_type=Release
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
	build_type=Debug
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	build_type=MinSizeRel
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OSPLAT=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osname`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

have_qt_build()
{
    # At this time, QT is only available for slf6
    osname=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osname`
    if [ "${osname}" != "slf6-x86_64" ]
    then
       echo "ERROR: the QT build of geant4 is only supported for slf6"
       exit 1
    fi
}

make_distribution()
{
    if [ "${maketar}" = "tar" ] 
    then
       if [ -d ${pkgdir}/lib64 ] || [ -d ${pkgdir}/lib ]
       then
	   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
	   exit 0
       fi
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=geant4
origpkgver=v4_10_1_p03
pkgver=${origpkgver}a
ssibuildshims_version=v0_09_02
cmake_version=v3_3_2
tarpatchver=`echo ${origpkgver} | cut -f 4 -d"_"`
tarminorver=`echo ${origpkgver} | cut -f 3 -d"_"`
tarmajorver=`echo ${origpkgver} | cut -f 1-2 -d"_"`
tarver=${tarmajorver}_0${tarminorver}_${tarpatchver}
g4subdir=Geant4-10.1.3
pkgdotver=`echo ${tarver} | sed -e 's/_/./g' | sed -e 's/^v//'`
#pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=geant${pkgdotver}.tar.gz
patchfile=fermi_geant`echo ${origpkgver} | sed -e 's/^v//'`.patch
#pkgbasedir=`echo ${origpkgver} | sed -e 's/^v4_//'| sed -e 's/_/./g' | sed -e 's/00/0/' | sed -e 's/p0//'`
#g4subdir=Geant4-${pkgbasedir}

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

make_distribution

echo "building ${package} for ${OSPLAT}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${basequal}" == e[79] ]]
then
  cxxflg="c++1y"
elif [[ "${basequal}" == e10 ]]
then
  cxxflg="c++1y"
elif [[ "${basequal}" == e[79]:qt  ]]
then
  cxxflg="c++1y"
  have_qt_build
elif [[ "${basequal}" == e10:qt  ]]
then
  cxxflg="c++1y"
  have_qt_build
else
  echo "ERROR: unrecognized basequal ${basequal}"
  usage
  exit 1
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
# make sure we start from an empty build directory
if [ -d ${blddir} ]; then rm -rf ${blddir}; fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# pull the geant4 datasets
${thisdir}/scripts/getG4DataSets.sh ${product_dir} || die "ERROR: failed to install datasets"

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || die "ERROR: fake declare failed"

set -x

cd ${sourcedir}
tar xzf ${tardir}/${pkgtarfile} || die "ERROR: failed to unwind the source."

# apply Fermi patches
patch -b -p0 < ${patchdir}/${patchfile} || die "ERROR: patch failed."

set +x

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed."

# We need cmake
setup -B cmake ${cmake_version} || die "ERROR: setup -B cmake ${cmake_version} failed"

# define use_qt here
if [ "${basequal}" = "e7:qt" ] || [ "${basequal}" = "e9:qt" ] || [ "${basequal}" = "e10:qt" ]
then
  if [ -z ${QT_FQ_DIR} ] 
  then
    echo "ERROR: QT_FQ_DIR is undefined, cannot make a QT build"
    exit 1
  fi
  use_qt="-DGEANT4_USE_QT=ON -DQT_QMAKE_EXECUTABLE=${QT_FQ_DIR}/bin/qmake"
else
  use_qt=""
fi

export GEANT4_SOURCE=${GEANT4_DIR}/source/geant${pkgdotver}
export BLDDIR=${blddir}
GEANT4_J=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${BLDDIR}
env CC=gcc CXX=g++ FC=gfortran cmake -DGEANT4_BUILD_CXXSTD="${cxxflg}" \
                                     -DCMAKE_INSTALL_PREFIX=${GEANT4_FQ_DIR} \
				     -DGEANT4_USE_SYSTEM_CLHEP=ON \
				     -DCLHEP_ROOT_DIR:STRING="${CLHEP_BASE_DIR}" \
				     -DBUILD_STATIC_LIBS=ON \
				     -DGEANT4_USE_OPENGL_X11=ON \
				     -DGEANT4_USE_GDML=ON \
				     ${use_qt} \
				     -DXERCESC_ROOT_DIR:STRING="${XERCESCROOT}" \
				     -DCMAKE_BUILD_TYPE=${build_type} \
				     ${GEANT4_SOURCE}
(( $? == 0 )) || die "ERROR: cmake failed."
make ${GEANT4_J:+-j ${GEANT4_J}} || die "ERROR: make failed."
make install  || die "ERROR: make install failed."

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed."

if [ ! -d ${G4LIB} ]
then
   echo "ERROR: failed to create ${G4LIB}"
   echo "       Something is very wrong"
   exit 1
fi
echo "G4LIB is ${G4LIB}"

# remove build directory from prof and opt distributions
if [ "${extraqual}" = "opt" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
elif [ "${extraqual}" = "prof" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
fi

# need to fix paths:
# bin/geant4-config
# lib64/Geant4-xxx/Geant4*.cmake
# share/Geant4-xxx/geant4make/geant4make*sh

fixfile=/tmp/g4fix$$$$.sh
rm -f ${fixfile}

echo "#!/bin/bash" > ${fixfile}
##echo " set -x " >>  ${fixfile}
echo "mv ${GEANT4_FQ_DIR}/bin/geant4-config ${GEANT4_FQ_DIR}/bin/geant4-config.bak" >> ${fixfile}
if [ -z ${QT_FQ_DIR} ] 
then
  echo "cat ${GEANT4_FQ_DIR}/bin/geant4-config.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%' | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${GEANT4_FQ_DIR}/bin/geant4-config" >> ${fixfile}
else
  echo "cat ${GEANT4_FQ_DIR}/bin/geant4-config.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%' | sed 's%${QT_FQ_DIR}%\${QT_FQ_DIR}%' | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${GEANT4_FQ_DIR}/bin/geant4-config" >> ${fixfile}
fi
echo "chmod +x  ${GEANT4_FQ_DIR}/bin/geant4-config" >> ${fixfile}

cmakeConfig=`ls ${G4LIB}/${g4subdir}/Geant4*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  if [ -z ${QT_FQ_DIR} ] 
  then
    echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%' | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%' > ${file}" >> ${fixfile}
  else
    echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%' | sed 's%${QT_FQ_DIR}%\${QT_FQ_DIR}%' | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%' > ${file}" >> ${fixfile}
  fi
done

g4make=`ls ${GEANT4_FQ_DIR}/share/${g4subdir}/geant4make/geant4make*sh`
for file in ${g4make}
do
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  if [ -z ${QT_FQ_DIR} ] 
  then
    echo "cat ${file}.bak | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%' | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${file}" >> ${fixfile}
  else
    echo "cat ${file}.bak | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%' | sed 's%${QT_FQ_DIR}%\${QT_FQ_DIR}%' | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${file}" >> ${fixfile}
  fi
  echo "chmod +x  ${file}" >> ${fixfile}
done

chmod +x ${fixfile}
${fixfile} || die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}

# this must be last
make_distribution

exit 0
